/**
   The command-line version of MRIConvert.
**/

#include <string>
#include <iostream>
#include <vector>

#include <wx/app.h>
#include <wx/cmdline.h>
#include <wx/string.h>
#include <wx/dir.h>
#include <wx/config.h>
#include <wx/stdpaths.h>

#include "McVerter.h"
#include "OutputFactory.h"
#include "Converter.h"
#include "OutputterBase.h"
#include "StringConvert.h"

using namespace std;
using namespace jcs;

extern std::string __OS_X_BUG_argv_0__;

int main(int argc, char** argv)
{
  // Due to problems with wxStandardPaths under OS X console, this is necessary.
  // See Dictionary.cpp - 20091015cdt
  __OS_X_BUG_argv_0__ = argv[0];

    // First make sure we can use wx stuff.
    wxInitializer initializer;
    if ( !initializer )
    {
        cerr << "Failed to initialize the wxWidgets library, aborting.";
        return -1;
    }

    // Parse command line arguments.
	wxString allowed_formats_string(_T("Output format:"));
	int type = 0;
	while(type < (OutputFactory::GetNumberOfTypes() - 1) ){
		allowed_formats_string.append(wxString(OutputFactory::GetShortDescription(type), wxConvLocal));
		allowed_formats_string.append(_T(", "));
		++type;
	}
	allowed_formats_string.append(_T("or "));
	allowed_formats_string.append(wxString(OutputFactory::GetShortDescription(type), wxConvLocal));

	static const wxCmdLineEntryDesc cmdLineDesc[] = {

		{ wxCMD_LINE_OPTION, _T("o"), _T("output"),
			_T("Output directory for converted files"),
			wxCMD_LINE_VAL_STRING, wxCMD_LINE_OPTION_MANDATORY  },
		{ wxCMD_LINE_OPTION, _T("f"), _T("format"), 
			allowed_formats_string,
			wxCMD_LINE_VAL_STRING, wxCMD_LINE_OPTION_MANDATORY },
		{ wxCMD_LINE_SWITCH, _T("x"), _T(""), _T("Save each series to a separate directory")},
		{ wxCMD_LINE_SWITCH, _T("d"), _T(""), _T("Save output volumes as 4D files")}, 
		{ wxCMD_LINE_SWITCH, _T("a"), _T(""), _T("Save header only (metaimage only)")},
		{ wxCMD_LINE_SWITCH, _T("n"), _T(""), _T("Save files as .nii (nifti/fsl only)")},
		{ wxCMD_LINE_SWITCH, _T("b"), _T(""), _T("Save .v16 files (BrainVoyager only)")},
		{ wxCMD_LINE_SWITCH, _T("u"), _T(""), _T("Use patient id instead of patient name for output file")},
		{ wxCMD_LINE_OPTION, _T("s"), _T("skip"), _T("Skip first n volumes"), wxCMD_LINE_VAL_NUMBER},
		{ wxCMD_LINE_OPTION, _T("m"), _T("match"), 
			_T("Only convert files whose series descriptions include this string")},
		{ wxCMD_LINE_SWITCH, _T("v"), _T(""), _T("Verbose mode")},
		{ wxCMD_LINE_SWITCH, _T("i"), _T(""), _T("Version info"),
			wxCMD_LINE_VAL_NONE, wxCMD_LINE_OPTION_HELP},
		{ wxCMD_LINE_SWITCH, _T("h"), _T("help"), _T("Display help"), 
			wxCMD_LINE_VAL_NONE, wxCMD_LINE_OPTION_HELP},
		{ wxCMD_LINE_PARAM, NULL, NULL, _T("Input directory or file(s)"),
			wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_MULTIPLE  },
		{ wxCMD_LINE_NONE }
	};

	wxCmdLineParser parser(argc, argv);

	parser.SetDesc(cmdLineDesc);

	if (parser.Parse() > 0) return 0;

	if (parser.Found(_T("i"))) {
		cout << "mcverter 2.0 rev. 190" << endl;
		return 0;
	}

	if (parser.Found(_T("help"))) return 0;

	wxString format;
	parser.Found(_T("format"), &format);
	int format_type = 0;
	int found = wxNOT_FOUND;

	while (format_type < OutputFactory::GetNumberOfTypes()) {
		found = format.Find(wxString(OutputFactory::GetShortDescription(format_type), wxConvLocal));
		if (found != wxNOT_FOUND) break;
		++format_type;
	}

	if (found == wxNOT_FOUND) {
		cerr << "Invalid format type: " << format << endl;
		cerr << allowed_formats_string << endl;
		return 0;
	}

	wxString outputDir;
	parser.Found(_T("output"), &outputDir);
	if (!wxDir::Exists(outputDir)) 
		if (!wxFileName::Mkdir(outputDir, 0777, wxPATH_MKDIR_FULL)) {
			cout << "Unable to create directory: " << outputDir << endl;
			return 0;
		}


	Converter* converter = new Converter(format_type);
	OutputterBase* outputter = converter->GetOutputter();

	outputter->mOutputList.rootDir = outputDir;

	outputter->SetOption("split", parser.Found(_T("x")));
	outputter->SetOption("dim", parser.Found(_T("d"))? 4 : 3);
	outputter->SetOption("nii", parser.Found(_T("n")));
	outputter->SetOption("ho", parser.Found(_T("a")));
	outputter->SetOption("v16", parser.Found(_T("b")));

	if (parser.Found(_T("u"))) {
		outputter->defaultNameFields[OutputterBase::PatientName] 
			= OutputterBase::NameField("PatientName", false);
		outputter->defaultNameFields[OutputterBase::PatientId]
			= OutputterBase::NameField("PatientId", true);
	}
	else {
		outputter->defaultNameFields[OutputterBase::PatientName] 
			= OutputterBase::NameField("PatientName", true);
		outputter->defaultNameFields[OutputterBase::PatientId]
			= OutputterBase::NameField("PatientId", false);
	}

	long skip = 0;
	parser.Found(_T("s"), &skip);
	outputter->SetOption("skip", static_cast<int> (skip));

	verbose = parser.Found(_T("v"));

	wxString match;
	parser.Found(_T("m"), &match);

	if (verbose) cout << "Adding files..." << endl;
	for (unsigned int i = 0; i < parser.GetParamCount(); ++i) {
		wxFileName filename(parser.GetParam(i));
		if (filename.FileExists()) {
			converter->AddFile(filename.GetFullPath(), match);
		}
		else if (filename.DirExists()) {
			wxArrayString files;
			wxDir::GetAllFiles(filename.GetFullPath(), &files);
			unsigned int n_files = files.GetCount();
			for (unsigned int i = 0; i < files.GetCount(); ++i) {
				converter->AddFile(files.Item(i), match);
			}
		}
		else cout << parser.GetParam(i) << " not recognized." << endl;
	}

	if (verbose) {
		MessageList::iterator ml_it = converter->messages.begin();
		while (ml_it != converter->messages.end()) {
			Message::iterator it = ml_it->begin();
			while (it != ml_it->end()) {
				cout << it->mb_str(wxConvLocal) << " ";
				++it;
			}
			cout << endl;
			++ml_it;
		}
	}
	converter->messages.clear();
	converter->UpdateAllOutput();

	if (verbose) cout << "Converting...." << endl;
	converter->ConvertAll();

	if (verbose) cout << "Finished" << endl;

	delete converter;

	return 0;

}


